module Gitlab
  module MarkupHelper
    module_function

    # Public: Determines if a given filename is compatible with GitHub::Markup.
    #
    # filename - Filename string to check
    #
    # Returns boolean
    def markup?(filename)
      gitlab_markdown?(filename) ||
        asciidoc?(filename) ||
        other_markup?(filename)
    end

    # Public: Determines if a given filename is compatible with
    # GitLab-flavored Markdown.
    #
    # filename - Filename string to check
    #
    # Returns boolean
    def gitlab_markdown?(filename)
      filename.downcase.end_with?(*%w(.mdown .mkd .mkdn .md .markdown))
    end

    # Public: Determines if the given filename has AsciiDoc extension.
    #
    # filename - Filename string to check
    #
    # Returns boolean
    def asciidoc?(filename)
      filename.downcase.end_with?(*%w(.adoc .ad .asciidoc))
    end

    # Public: Determines if the given filename is plain text.
    #
    # filename - Filename string to check
    #
    # Returns boolean
    def plain?(filename)
      filename.downcase.end_with?('.txt') ||
        filename.casecmp('readme').zero?
    end

    def other_markup?(filename)
      filename.downcase.end_with?(*%w(.textile .rdoc .org .creole .wiki
                                      .mediawiki .rst))
    end

    def previewable?(filename)
      markup?(filename)
    end

    # Public: Determines markup language from the given filename.
    #
    #   Markdown         => :md
    #   AsciiDoc         => :adoc
    #   Textile          => :textile
    #   RDoc             => :rdoc
    #   Org              => :org
    #   Creole           => :creole
    #   MediaWiki        => :mediawiki
    #   reStructuredText => :rst
    #   plain            => :plain
    #   unknown          => nil
    #
    # filename - Filename string to check
    #
    # Returns Symbol or nil
    def markup_lang(filename)
      if gitlab_markdown?(filename)
        :md
      elsif asciidoc?(filename)
        :adoc
      elsif plain?(filename)
        :plain
      elsif other_markup?(filename)
        name = File.extname(filename.downcase)[1..-1]
        name.to_sym if name != 'wiki'
      end
    end
  end
end
